function [est,se]=cic(Y00,Y01,Y10,Y11,qq,standard_error,bootstrap);

% GENERAL INFORMATION
% this program calculates the four sets of cic estimates
% (for the continuous, discrete conditional independence, lower and upper bound)
% if required it also calculates analytic standard errors and bootstrap standard errors

% INPUT
% Y00 is vector of outcomes in (0,0) group
% Y01 is vector of outcomes in (0,1) group
% Y10 is vector of outcomes in (1,0) group
% Y11 is vector of outcomes in (1,1) group
% qq is a vector of quantiles for which the quantile effects will be calculated
% standard_error is 0/1 indicator whether standard errors should be calculated
% bootstrap is an indicator whether bootstrap standard errors should be calculated
% if positive then bootstrap standard errors will be calculated with
% number of bootstrap replications equal to the value of bootstrap.

% OUTPUT
% est is 4-dimensional vector with estimates
% 1. continuous
% 2. discrete with conditional independence
% 3. lower bound
% 4. upper bound
% se is 4-dimensional vector with estimates of standard errors
% 1. continuous
% 2. discrete with conditional independence
% 3. lower bound
% 4. upper bound

% preliminary manipulation of the data
YS=supp([Y00;Y01;Y10;Y11]); % vector of distinct support points
NYS=length(YS);             % number of points of support           
YS00=supp(Y00);             % vector of distinct support points for Y00
NYS00=length(YS00);
YS01=supp(Y01);             % vector of distinct support points for Y01
NYS01=length(YS01);
YS10=supp(Y10);             % vector of distinct support points for Y10
NYS10=length(YS10);
YS11=supp(Y11);             % vector of distinct support points for Y11
NYS11=length(YS11);
f00=prob(Y00,YS);           % vector of probabilities
f01=prob(Y01,YS);           % vector of probabilities
f10=prob(Y10,YS);           % vector of probabilities
f11=prob(Y11,YS);           % vector of probabilities

% continuous estimate
est_con=cic_con(f00,f01,f10,f11,qq,YS,YS01);

% conditional independence estimate
est_dci=cic_dci(f00,f01,f10,f11,qq,YS,YS01);

% lower bound estimate
est_low=cic_low(f00,f01,f10,f11,qq,YS,YS01);

% upper bound estimate
est_upp=cic_upp(f00,f01,f10,f11,qq,YS,YS01);

est=[est_con,est_dci,est_low,est_upp];

% standard error calculations
se=0*est;

% standard error for conditional independence
% this standard error is based on numerical differentiation
% in combination with the delta method.
% the estimator is a function of f00, f01, f10, f11.

if standard_error==1,
   F00=cumsum(f00); % cumulative distribution functions
   F01=cumsum(f01); % cumulative distribution functions
   F10=cumsum(f10); % cumulative distribution functions
   F11=cumsum(f11); % cumulative distribution functions
   F00=F00/F00(length(F00),1);  % ensuring normalization to one
   F01=F01/F01(length(F01),1);
   F10=F10/F10(length(F10),1);
   F11=F11/F11(length(F11),1);
   
   % A. continuous estimator
   % 0. preliminaries
   cc=0.00000001;
   F00_10=zeros(NYS10,1);
   F01invF00_10=zeros(NYS10,1);
   for i=1:NYS10;
       F00_10(i,1)=cdf(YS10(i,1),F00,YS);
       F01invF00_10(i,1)=cdfinv(F00_10(i,1),F01,YS);
       f01F01invF00_10(i,1)=fden(F01invF00_10(i,1),Y01);
       end
   % 1. contribution of Y00
   P=zeros(NYS00,1);
   for i=1:NYS00,
       PY00=((YS00(i,1)<=YS10)-F00_10)./f01F01invF00_10;
       P(i,1)=PY00'*f10(f10>cc);
       end,
   V00=sum(P.*P.*f00(f00>cc))/length(Y00);
   % 2. contribution of Y01
   P=zeros(NYS01,1);
   for i=1:NYS01,
       PY01=-((cdf(YS01(i,1),F01,YS)<=F00_10)-F00_10)./f01F01invF00_10;
       P(i,1)=PY01'*f10(f10>cc);
       end,
   V01=sum(P.*P.*f01(f01>cc))/length(Y01);
   % 3. contribution of Y10
   P=F01invF00_10-F01invF00_10'*f10(f10>cc);
   V10=sum(P.*P.*f10(f10>cc))/length(Y10);
   % 4. contribution of Y11
   P=YS11-YS'*f11;
   V11=sum(P.*P.*f11(f11>cc))/length(Y11);
   se_con=sqrt(V00+V01+V10+V11);
   se(1,1)=se_con;       
   
   % B. dci standard error
   % numerical approximation to delta method
   % four parts to variance
   cc=0.00000001;
   delta=0.0000001;
   % 1. contribution of Y00
   der00=zeros(NYS,1);
   I00=f00>cc;
   NYS00=sum(I00);
   t=(1:NYS)';
   tI00=t(I00,1);
   max00=max(tI00);
   V00=(diag(f00)-f00*(f00'))/length(Y00);
   V00(max00,:)=zeros(1,NYS);
   V00(:,max00)=zeros(NYS,1);
   for i=1:NYS00-1,
       f00c=f00;
       f00c(tI00(i),1)=f00c(tI00(i),1)+delta;
       f00c(max00,1)=f00c(max00,1)-delta;
       der00_c=cic_dci(f00c,f01,f10,f11,qq,YS,YS01);
       der00(tI00(i,1),1)=(der00_c(1,1)-est_dci(1,1))/delta;
       end,
   V00=der00'*V00*der00;   
   % 2. Contribution of Y01
   der01=zeros(NYS,1);
   I01=f01>cc;
   NYS01=sum(I01);
   t=(1:NYS)';
   tI01=t(I01,1);
   max01=max(tI01);
   V01=(diag(f01)-f01*(f01'))/length(Y01);
   V01(max01,:)=zeros(1,NYS);
   V01(:,max01)=zeros(NYS,1);
   for i=1:NYS01-1,
       f01c=f01;
       f01c(tI01(i),1)=f01c(tI01(i),1)+delta;
       f01c(max01,1)=f01c(max01,1)-delta;
       der01_c=cic_dci(f00,f01c,f10,f11,qq,YS,YS01);
       der01(tI01(i,1),1)=(der01_c(1,1)-est_dci(1,1))/delta;
       end,
   V01=der01'*V01*der01;
   % 3. Contribution of Y10
   der10=zeros(NYS,1);
   I10=f10>cc;
   NYS10=sum(I10);
   t=(1:NYS)';
   tI10=t(I10,1);
   max10=max(tI10);
   V10=(diag(f10)-f10*(f10'))/length(Y10);
   V10(max10,:)=zeros(1,NYS);
   V10(:,max10)=zeros(NYS,1);
   for i=1:NYS10-1,
       f10c=f10;
       f10c(tI10(i),1)=f10c(tI10(i),1)+delta;
       f10c(max10,1)=f10c(max10,1)-delta;
       der10_c=cic_dci(f00,f01,f10c,f11,qq,YS,YS01);
       der10(tI10(i,1),1)=(der10_c(1,1)-est_dci(1,1))/delta;
       end,
   V10=der10'*V10*der10;   
   % 4. Contribution of Y11
   der11=zeros(NYS,1);
   I11=f11>cc;
   NYS11=sum(I11);
   t=(1:NYS)';
   tI11=t(I11,1);
   max11=max(tI11);
   V11=(diag(f11)-f11*(f11'))/length(Y11);
   V11(max11,:)=zeros(1,NYS);
   V11(:,max11)=zeros(NYS,1);
   for i=1:NYS11-1,
       f11c=f11;
       f11c(tI11(i),1)=f11c(tI11(i),1)+delta;
       f11c(max11,1)=f11c(max11,1)-delta;
       der11_c=cic_dci(f00,f01,f10,f11c,qq,YS,YS01);
       der11(tI11(i,1),1)=(der11_c(1,1)-est_dci(1,1))/delta;
       end,
   V11=der11'*V11*der11;   
   NNN=length(Y00)+length(Y01)+length(Y10)+length(Y11);
   N00=length(Y00);
   alpha00=N00/NNN;
   N01=length(Y01);
   alpha01=N01/NNN;
   N10=length(Y10);
   alpha10=N10/NNN;
   N11=length(Y11);
   alpha11=N11/NNN;
   [V00*N00,V01*N01,V10*N10,V11*N11];
   %'components dci variance'
   se_dci=sqrt(V00+V01+V10+V11);
   se(1,2+length(qq))=se_dci;
   
   % C. lower bound standard error
   k_bar=zeros(NYS10,1);
   for i=1:NYS10,
       F00_10=cdf(YS10(i,1),F00,YS);
       F001invF00_10=cdfinv(F00_10,F01,YS);
       k_bar(i,1)=F001invF00_10; 
       end
   k_bar=k_bar-k_bar'*f10(f10>cc);
   V10=sum(k_bar.*k_bar.*f10(f10>cc))/length(Y10);
   dy11=YS11-YS11'*f11(f11>cc);
   V11=sum(dy11.*dy11.*f11(f11>cc))/length(Y11);
   se_lb=sqrt(V10+V11);
   se(1,3+2*length(qq))=se_lb;

   % D. upper bound standard error
   k_bar=zeros(NYS10,1);
   for i=1:NYS10,
       F00b_10=cdf_bar(YS10(i,1),F00,YS);
       F01invF00_10=cdfinv(F00b_10,F01,YS);
       k_bar(i,1)=F01invF00_10;
       end
   k_bar=k_bar-k_bar'*f10(f10>cc);
   V10=sum(k_bar.*k_bar.*f10(f10>cc))/length(Y10);
   dy11=YS11-YS11'*f11(f11>cc);
   V11=sum(dy11.*dy11.*f11(f11>cc))/length(Y11);
   se_ub=sqrt(V10+V11);
   se(1,4+3*length(qq))=se_ub;
   end

 
if bootstrap>0,
   boot=0*est;
   Nboot=bootstrap;
   boot_est=zeros(Nboot,4);
   N00=length(Y00);
   N01=length(Y01);
   N10=length(Y10);
   N11=length(Y11);
   for i=1:Nboot;
       u=floor(N00*rand(N00,1)+1);
       Y00b=Y00(u,1);
       u=floor(N01*rand(N01,1)+1);
       Y01b=Y01(u,1);
       u=floor(N10*rand(N10,1)+1);
       Y10b=Y10(u,1);
       u=floor(N11*rand(N11,1)+1);
       Y11b=Y11(u,1);
       YSb=supp([Y00b;Y01b;Y10b;Y11b]);
       YS01b=supp(Y01b);
       f00b=prob(Y00b,YSb); % vector of probabilities
       f01b=prob(Y01b,YSb); % vector of probabilities
       f10b=prob(Y10b,YSb); % vector of probabilities
       f11b=prob(Y11b,YSb); % vector of probabilities
       boot_est(i,1:1+length(qq))=cic_con(f00b,f01b,f10b,f11b,qq,YSb,YS01b);
       boot_est(i,length(qq)+2:2+2*length(qq))=cic_dci(f00b,f01b,f10b,f11b,qq,YSb,YS01b);
       boot_est(i,3+2*length(qq):3+3*length(qq))=cic_low(f00b,f01b,f10b,f11b,qq,YSb,YS01b);
       boot_est(i,4+3*length(qq):4+4*length(qq))=cic_upp(f00b,f01b,f10b,f11b,qq,YSb,YS01b);
       se_boot=std(boot_est);
       end    
   if Nboot>=100,                  % eventually bootstrap standard errors are 
                                   % calculated using the percentile method
      low=round(Nboot*0.025);
      high=round(Nboot*0.975);
      boot_est=sort(boot_est);
      se_boot=(boot_est(high,:)-boot_est(low,:))/(2*1.96);
      end
   se=[se;se_boot];   
   end
    